import React, {
    Component
} from 'react';
import PropTypes from 'prop-types';
import {
    ViewPropTypes,
    StyleSheet,
    View,
    Text,
    Dimensions,
    TouchableWithoutFeedback,
    Easing,
    Keyboard,
    Animated,
    Image,
    ImageBackground,
    Platform
} from 'react-native';
import * as Animatable from 'react-native-animatable';
import {widthPercentageToDP as wp, heightPercentageToDP as hp} from 'react-native-responsive-screen';
import { Col, Row, Grid } from "react-native-easy-grid";
const window = Dimensions.get('window');
const TOAST_MAX_WIDTH = 1;
const TOAST_ANIMATION_DURATION = 200;
const radioDevice = window.width*window.height;
const positions = {
    TOP: 1,
    BOTTOM: -20,
    CENTER: 0
};
import GLOBAL from '../../../services/Global';

const durations = {
    LONG: 4000,
    SHORT: 2000
};

let styles = StyleSheet.create({
    defaultStyle: {
        position: 'absolute',
        left: 0,
        right: 0,
        top: 0,
        width: wp('100%'),
        justifyContent: 'center',
        alignItems: 'center'
    },
    containerStyle: {
        padding: 0,
        opacity: 1,
        backgroundColor: 'transparent',
        borderRadius: 0,
        width: wp('100%')
    },
    imgErrores: {
      width: 160,
      height: 230,
      position: 'absolute',
      bottom: Platform.Version === 19 || Platform.Version === 17 || Platform.Version === 18 ? 46 : 34,
      zIndex: 10,
      resizeMode: 'cover',
    },
    imgErroresCelular: {
      width: 70,
      height: 140,
      position: 'absolute',
      bottom: radioDevice < 220000 ? 16 : 18,
      zIndex: 10,
      resizeMode: 'cover',
    },
    shadowStyle: {
      shadowColor: "#000",
      shadowOffset: {
       width: 0,
       height: 12,
      },
      shadowOpacity: 0.58,
      shadowRadius: 16.00,
      elevation: 24,
    },
    contenedorMSG: {
      paddingTop: 0,
      paddingLeft: 15,
      paddingRight: 15
    },
    subTitle: {
      fontSize: window.width > 500 ? hp('2.5%') : hp('2%'),
      fontStyle: 'normal',
      color: 'white',
    },
    bgToast: {
      marginTop: -70,
      zIndex: 1,
      height: 240,
      width: wp('103%'),
      position: 'relative'
    },
    bgToastTablet: {
      zIndex: 1,
      marginLeft: -18,
      marginTop: Platform.Version === 19 || Platform.Version === 17 || Platform.Version === 18 ? -200 : -120,
      height: Platform.Version === 19 || Platform.Version === 17 || Platform.Version === 18 ? 550 : 420,
      width: wp('106%'),
      position: 'relative'
    },
    textStyle: {
      fontSize: window.width > 500 ? hp('2.8%') : hp('3%'),
      fontStyle: 'normal',
      fontFamily: Platform.OS === 'ios' ? GLOBAL.FONTFAMILY : GLOBAL.FONTFAMILYANDROID,
      color: 'white'
        // paddingTop: window.width > 500 ? Platform.Version === 19 || Platform.Version === 17 || Platform.Version === 18 ? 100 : 65 : 70,
        // paddingRight: window.width > 500 ? 20 : 0,
    },
    titleChildren: {
      fontSize: window.width > 500 ? hp('3.8%') : hp('3%'),
      fontStyle: 'normal',
      fontFamily: Platform.OS === 'ios' ? GLOBAL.FONTFAMILY : GLOBAL.FONTFAMILYANDROID,
      color: 'white',
      paddingBottom: 8,
      paddingTop: 5
    }
});

class ToastContainer extends Component {
    static displayName = 'ToastContainer';

    static propTypes = {
        ...ViewPropTypes,
        containerStyle: ViewPropTypes.style,
        duration: PropTypes.number,
        visible: PropTypes.bool,
        position: PropTypes.number,
        animation: PropTypes.bool,
        shadow: PropTypes.bool,
        keyboardAvoiding: PropTypes.bool,
        backgroundColor: PropTypes.string,
        opacity: PropTypes.number,
        shadowColor: PropTypes.string,
        textColor: PropTypes.string,
        textStyle: Text.propTypes.style,
        delay: PropTypes.number,
        hideOnPress: PropTypes.bool,
        onPress: PropTypes.func,
        onHide: PropTypes.func,
        onHidden: PropTypes.func,
        onShow: PropTypes.func,
        onShown: PropTypes.func,
        tipo: PropTypes.string,
        subtitulo: PropTypes.string
    };

    static defaultProps = {
        visible: false,
        duration: durations.SHORT,
        animation: false,
        shadow: true,
        position: positions.BOTTOM,
        opacity: 1,
        delay: 0,
        hideOnPress: true,
        keyboardAvoiding: true
    };

    constructor() {
        super(...arguments);
        const window = Dimensions.get('window');
        this.state = {
            visible: this.props.visible,
            opacity: new Animated.Value(0),
            windowWidth: window.width,
            windowHeight: window.height,
            keyboardScreenY: window.height,
            isShown: true
        };
    }

    componentWillMount() {

        Dimensions.addEventListener('change', this._windowChanged);
        if (this.props.keyboardAvoiding) {
            Keyboard.addListener('keyboardDidChangeFrame', this._keyboardDidChangeFrame);
        }
    }

    componentDidMount = () => {


        if (this.state.visible) {
            this._showTimeout = setTimeout(() => this._show(), this.props.delay);
        }
    };

    componentWillReceiveProps = nextProps => {

        if (nextProps.visible !== this.props.visible) {
            if (nextProps.visible) {
                clearTimeout(this._showTimeout);
                clearTimeout(this._hideTimeout);
                this._showTimeout = setTimeout(() => this._show(), this.props.delay);
            } else {
                this._hide();
            }

            this.setState({
                visible: nextProps.visible
            });
        }
    };

    componentWillUnmount = () => {

        this._hide();
        Dimensions.removeEventListener('change', this._windowChanged);
        Keyboard.removeListener('keyboardDidChangeFrame', this._keyboardDidChangeFrame);
    };

    _animating = false;
    _root = null;
    _hideTimeout = null;
    _showTimeout = null;
    _keyboardHeight = 0;

    _windowChanged = ({ window }) => {

        this.setState({
            windowWidth: window.width,
            windowHeight: window.height
        })
    };

    _keyboardDidChangeFrame = ({ endCoordinates }) => {
        this.setState({
            keyboardScreenY: endCoordinates.screenY
        })
    };

    _show = () => {

        clearTimeout(this._showTimeout);
        if (!this._animating) {
            clearTimeout(this._hideTimeout);
            this._animating = true;
            this._root.setNativeProps({
                pointerEvents: 'auto'
            });
            this.props.onShow && this.props.onShow(this.props.siblingManager);
            Animated.timing(this.state.opacity, {
                toValue: this.props.opacity,
                duration: this.props.animation ? TOAST_ANIMATION_DURATION : 0,
                easing: Easing.out(Easing.ease)
            }).start(({finished}) => {
                if (finished) {
                    this._animating = !finished;
                    this.props.onShown && this.props.onShown(this.props.siblingManager);
                    if (this.props.duration > 0) {
                        this._hideTimeout = setTimeout(() => this._hide(), this.props.duration);
                    }
                }
            });
        }
    };

    _hide = () => {
      this.shakeIn;
        clearTimeout(this._showTimeout);
        clearTimeout(this._hideTimeout);
        if (!this._animating) {
            if (this._root) {
                this._root.setNativeProps({
                    pointerEvents: 'none'
                });
            }

            if (this.props.onHide) {
                this.props.onHide(this.props.siblingManager);
            }

            Animated.timing(this.state.opacity, {
                toValue: 0,
                duration: this.props.animation ? TOAST_ANIMATION_DURATION : 0,
                easing: Easing.in(Easing.ease)
            }).start(({finished}) => {
                if (finished) {
                  this.setState({
                      isShown: false
                  });
                    this._animating = false;
                    this.props.onHidden && this.props.onHidden(this.props.siblingManager);
                }
            });
        }
    };

    render() {
        let {props} =  this;
        const { windowWidth } = this.state;
        let offset = props.position;

        const { windowHeight, keyboardScreenY } = this.state;
        const keyboardHeight = Math.max(windowHeight - keyboardScreenY, 0);
        let position = offset ? {
            [offset < 0 ? 'bottom' : 'top']: offset < 0 ? (keyboardHeight - offset) : offset
        } : {
            top: 0,
            bottom: keyboardHeight
        };

        if(this.props.tipo == 'sinpremio'){
          return (this.state.visible || this._animating) ? <View
              style={[
                  styles.defaultStyle,
                  position
              ]}
              pointerEvents="box-none"
          >
              <TouchableWithoutFeedback
                  onPress={() => {
                    typeof this.props.onPress === 'function' ? this.props.onPress() : null
                    this.props.hideOnPress ? this._hide() : null
                  }}
              >
                  <Animatable.View
                    animation={this.state.isShown ? 'bounceInDown' : 'bounceOutUp'} duration={1900}
                      pointerEvents="none"
                      ref={ele => this._root = ele}
                  >
                  <ImageBackground
                    resizeMode={'contain'}
                    style={window.width > 500 ? styles.bgToastTablet : styles.bgToast }
                    source={require('../../../images/bggeos.png')}>
                  <Grid>
                    <Col style={{width: '40%', justifyContent: 'center', alignItems: 'center'}}>

                    </Col>
                    <Col style={{justifyContent: 'center', alignItems: 'center'}}>
                      <View style={styles.contenedorMSG}>
                        <Text style={styles.titleChildren}>
                            {this.props.children}
                        </Text>
                        <Text style={styles.subTitle}>
                          {this.props.subtitulo}
                        </Text>
                      </View>
                    </Col>
                  </Grid>
                  </ImageBackground>
                  </Animatable.View>
              </TouchableWithoutFeedback>
          </View> : null;
        }



        if(this.props.tipo == 'sintrivias'){
          return (this.state.visible || this._animating) ? <View
              style={[
                  styles.defaultStyle,
                  position
              ]}
              pointerEvents="box-none"
          >
              <TouchableWithoutFeedback
                  onPress={() => {
                    typeof this.props.onPress === 'function' ? this.props.onPress() : null
                    this.props.hideOnPress ? this._hide() : null
                  }}
              >
                  <Animatable.View
                    animation={this.state.isShown ? 'bounceInDown' : 'bounceOutUp'} duration={1900}
                      pointerEvents="none"
                      ref={ele => this._root = ele}
                  >
                  <ImageBackground
                    resizeMode={'contain'} // or cover
                    style={window.width > 500 ? styles.bgToastTablet : styles.bgToast }
                    source={require('../../../images/bgcris.png')}>
                  <Grid>
                    <Col style={{width: '40%', justifyContent: 'center', alignItems: 'center'}}>

                    </Col>
                    <Col style={{justifyContent: 'center', alignItems: 'center'}}>
                      <View style={styles.contenedorMSG}>
                        <Text style={styles.titleChildren}>
                            {this.props.children}
                        </Text>
                        <Text style={styles.subTitle}>
                          {this.props.subtitulo}
                        </Text>
                      </View>
                    </Col>
                  </Grid>
                  </ImageBackground>
                  </Animatable.View>
              </TouchableWithoutFeedback>
          </View> : null;
        }
    }
}

export default ToastContainer;
export {
    positions,
    durations
}
